/******************************************************************************\
 * addfoldericons: ConcurrentPathProcessor.h
 *
 * Derive a class from NSOperation which can be used to concurrently process a
 * full POSIX path to a folder in order to update that folder's icon. The class
 * may be run by, for example, adding it as an operation to an NSOperationQueue
 * instance.
 *
 * (C) Hipposoft 2009 <ahodgkin@rowing.org.uk>
 \******************************************************************************/

#import <Cocoa/Cocoa.h>

@interface ConcurrentPathProcessor : NSOperation
{
    NSString   * pathData;
    CGImageRef   backgroundRef;
}

/* Initialise the class by passing a full POSIX path to the folder of interest
 * and a background image or NULL. For more on this second parameter, see the
 * documentation in the library code for the "backgroundImage" parameter of
 * "generateThumbnailForFolder" in "ThumbnailGenerator.[h|m]".
 */

-( id ) initWithPathAndBackground : ( NSString * ) fullPosixPath
                                  : ( CGImageRef ) backgroundImage;

@end /* @interface ConcurrentPathProcessor : NSOperation */
