/******************************************************************************\
 * addfoldericon - command line program to assign an icon to a folder
 * which gives an indication of any images held inside that folder, or
 * any of its sub-folders.
 *
 * (C) Hipposoft 2009 <ahodgkin@rowing.org.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

#include <stdio.h>
#include <sysexits.h>

#import <Cocoa/Cocoa.h>

/* Application functions and data */

#import "GlobalConstants.h"
#import "ConcurrentPathProcessor.h"

Boolean globalErrorFlag = NO; /* See GlobalConstants.h */

/* Library functions and data */

#import "Icons.h"
#import "GlobalSemaphore.h"

/* Local functions */

static void printHelp    ( void );
static void printVersion ( void );

/******************************************************************************\
 * First thing's first...
\******************************************************************************/

int main( int argc, const char * argv[] )
{
    /* See if a correct number of arguments was submitted - if so, read
     * the argument data, else complain.
     */

    if ( argc < 2 )
    {
        printVersion();
        printHelp();
        return EX_USAGE;
    }

    /* Prerequisites */
    
    NSAutoreleasePool * pool            = [ [ NSAutoreleasePool alloc ] init ];
    NSOperationQueue  * queue           = [ [ NSOperationQueue  alloc ] init ];
    CGImageRef          backgroundImage = getFolderIcon();
    
    globalSemaphoreInit();

    /* Process pathnames and add Grand Central Dispatch operations for each */
    
    for ( size_t i = 1; i < argc; i ++ )
    {
        NSString * fullPosixPath =
        [
            NSString stringWithUTF8String: argv[ i ]
        ];

        ConcurrentPathProcessor * processThisPath =
        [
            [ ConcurrentPathProcessor alloc ]
            initWithPathAndBackground: fullPosixPath
                                     : backgroundImage
        ];
        
        [ fullPosixPath release ];

        NSArray * oneOp = [ NSArray arrayWithObject: processThisPath ];
        [ queue addOperations: oneOp waitUntilFinished: NO ];
    }

    [ queue waitUntilAllOperationsAreFinished ];

    [ queue release ];
    [ pool  release ];

    return ( globalErrorFlag == YES ) ? EXIT_FAILURE : EXIT_SUCCESS;
}

/******************************************************************************\
 * Print version and author to stdout,
\******************************************************************************/

static void printVersion (void)
{
    printf("%s version %s by %s\n", PROGRAM_STRING, VERSION_STRING, AUTHOR_STRING);
}

/******************************************************************************\
 * Print help string to stdout,
\******************************************************************************/

static void printHelp (void)
{
    printf("Usage: %s path_to_folder\n", PROGRAM_STRING);
}
